#!/bin/sh
# Common ML Driver (CMLD)
#	Copyright 1993,1994,1995,1996 (C) SAITOH-Ely-Yasuhiro.
#	All rights reserved.
#
VERSION=@VER@
VHEAD="X-ML-Driver: CMLD (Version $VERSION)"

#
# Set path of sendmail
#
SENDMAIL=@SENDMAIL@

#
# Set local environment.
#
DRIVER=@DRIVER@
DOMAIN=@DOMAIN@
ORGDOM=@DOMAIN@
MSPOOL=@MSPOOL@
PATH=@PATH@
AWK=@AWK@
GREP=@GREP@
export DRIVER DOMAIN ORGDOM MSPOOL PATH
umask 022

# Get My PID.
MYPID=$$
export MYPID

# Get name of Mailing List.
ML=$1
export ML

LOOPCHECK="X-Loop-Check: $ML@$DOMAIN"

# Set work file.
HEAD=/tmp/HEAD.$$
BODY=/tmp/BODY.$$
TEMP=/tmp/TEMP.$$
WORK=/tmp/WORK.$$
ORIG=/tmp/ORIG.$$
export HEAD BODY TEMP WORK ORIG

# File for lock system.
# 1. For seq. file.
SEQFILE=$MSPOOL/$ML/spool/.seq
SEQLOCK=$MSPOOL/$ML/spool/.seq.lock
# 2. For digest format files
DGSTFILE=$MSPOOL/$ML/spool/.digest
DGSTLOCK=$MSPOOL/$ML/spool/.digest.lock
# 3. For change spool dir
SLKEY=$MSPOOL/$ML/spool/.SLOCK
SLOCK=$MSPOOL/$ML/spool/.spool.lock

trap "rm -f $HEAD $BODY $TEMP $WORK $ORIG $SEQLOCK $DGSTLOCK $SLOCK; exit 0" 0 1 2 3 15

# Lock key.
LKEY=$MSPOOL/$ML/spool/.DLOCK

# make lock key
if [ ! -f $LKEY ]
then
	touch $LKEY
fi
if [ ! -f $SLKEY ]
then
	touch $SLKEY
fi

# Spool dir
SPOOL=$MSPOOL/$ML/spool

# Member list file
LIST=$DRIVER/$ML/$ML-member

# List of PostOnlt Member
POST=$DRIVER/$ML/$ML-postonly

# Mail deliver list
DELV=$DRIVER/$ML/$ML-list

if [ ! -f $LIST ]
then
	# Copy list
	cat $DELV > $LIST

	# Convert list
	egrep "^[0-9]* SEND" $LIST | sed -e 's/.*<//' -e 's/>.*//' > $DELV

	if [ ! "`cat $DELV`" ]
	then
		cat $LIST > $DELV
	fi
fi

# Set default value.
. $DRIVER/config.default

if [ -f $DRIVER/$ML/config.$ML ]
then
	. $DRIVER/$ML/config.$ML
fi

if [ ! "$MESSAGES" ]
then
	MESSAGES=messages
fi

# Check change domain name.
if [ "$GENERICFROM" ]
then
	DOMAIN=$GENERICFROM
fi

# Make unix-from address.
UnixFrom=$ML-request@$DOMAIN

# Save original mail.
cat > $ORIG

# Delete 'Received:' header and save.
$AWK '
BEGIN {rcv = 0;}
# body
/^$/, NR == 0 {print; next;}
#header
/^From / {next;}
/^[Rr][Ee][Cc][Ee][Ii][Vv][Ee][Dd]:/ {rcv = 1; next;}
/^[ 	]/ { if( rcv == 1) {next;} }
{rcv = 0;}
/^([Ee][Rr][Rr][Oo][Rr][Ss]-[Tt][Oo]|[Rr][Ee][Tt][Uu][Rr][Nn]-[Rr][Ee][Cc][Ee][Ii][Pp][Tt]-[Tt][Oo]):/ {print "X-" $0; next;}
{print;};'< $ORIG > $TEMP

# Check header (loop)
sed -n -e '1,/^$/p' < $TEMP | $GREP -i "$LOOPCHECK" >/dev/null 2>&1
if [ $? -eq 0 ]
then
	echo "To: $ML-request@$DOMAIN ($ML ML admin)"		       > $HEAD
	echo "From: $ML-request@$DOMAIN ($ML ML admin)"		      >> $HEAD
	echo "Subject: Maybe already send this mail."		      >> $HEAD
	echo "Errors-To: $ML-request@$DOMAIN"			      >> $HEAD
	echo "$VHEAD"						      >> $HEAD

	if [ "$SENDER" ]
	then
		echo "Sender: $SENDER"				      >> $HEAD
	fi

	if [ "$PRECEDENCE" ]
	then
		echo "Precedence: $PRECEDENCE"			      >> $HEAD
	fi
	echo ""							      >> $HEAD
	echo "***********************************"		      >> $HEAD
	echo "** Maybe already send this mail. **"		      >> $HEAD
	echo "** Check original.               **"		      >> $HEAD
	echo "***********************************"		      >> $HEAD
	echo "------------------------------------------------------" >> $HEAD

	cat $HEAD $ORIG | $SENDMAIL -oi -t -f$UnixFrom
	exit 0
fi

# Check member.
if [ $NOTMEMBER -ne 1 ]
then
	FROM=`$AWK '
		BEGIN {rcv = 0;}
		/^$/, NR == 0 { next;}
		/^From / {next;}
		/^[Ff][Rr][Oo][Mm]:/ {rcv = 1; print; next;}
		/^[ 	]/ && rcv == 1 {print; next;}
		{rcv = 0};' < $TEMP`

	TFROM=`echo $FROM | tr -d '\012'`
	FROM=`echo $TFROM | sed -e 's/[Ff][Rr][Oo][Mm]: //' \
	-e 's/(.*)//' -e 's/.*<//' -e 's/>.*//' \
	-e 'y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/'`

	set $FROM
	FROM=$1

	echo $FROM | $GREP "@" >/dev/null 2>&1
	if [ $? -ne 0 ]
	then
		FROM="$FROM@$ORGDOM"
	fi

	MEMB=1
	if [ "$GENERIC" = "DONT" ]
	then
		cat $LIST $POST | sed -e 's/</(/g' -e 's/>/)/g' | \
			fgrep -i "($FROM)" >/dev/null 2>&1
		if [ $? -eq 0 ]
		then
			MEMB=0
		fi
	else
		FUSR=`echo $FROM | sed -e 's/@.*//'`
		FDOM=`echo $FROM | sed -e 's/.*@//'`

		for i in `cat $LIST $POST | sed -e 's/</(/g' -e 's/>/)/g' | \
					fgrep -i "($FUSR@" |\
					sed -e 's/.*(//' -e 's/).*//'`
		do
			LDOM=`echo "($i)" | sed -e 's/.*@//'`
			echo "$FDOM)" | fgrep -i "$LDOM" >/dev/null 2>&1
			if [ $? -eq 0 ]
			then
				MEMB=0
				break
			fi
		done
	fi

	set `echo $ML-request@$DOMAIN | \
	sed -e 'y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/'`
	ADMIN=$1

	if [ "$FROM" = "$ADMIN" ]
	then
		MEMB=0
	fi

	if [ $MEMB -ne 0 ]
	then
		echo "To: $FROM"				       > $HEAD
		if [ $NOTMEMBER -eq 3 ]
		then
			echo "Cc: $ML-request@$DOMAIN ($ML ML admin)" >> $HEAD
		fi
		echo "From: $ML-request@$DOMAIN ($ML ML admin)"	      >> $HEAD
		echo "Subject: You can not send to $ML ML."	      >> $HEAD
		echo "Errors-To: $ML-request@$DOMAIN"		      >> $HEAD
		echo "$VHEAD"					      >> $HEAD

		if [ "$SENDER" ]
		then
			echo "Sender: $SENDER"			      >> $HEAD
		fi

		if [ "$PRECEDENCE" ]
		then
			echo "Precedence: $PRECEDENCE"		      >> $HEAD
		fi

		echo ""						      >> $HEAD

		cat $HEAD $DRIVER/$MESSAGES/notmemb $ORIG | \
			sed -e 's/^[Ff][Rr][Oo][Mm][ 	]/\>From /' | \
			$SENDMAIL -oi -t -f$UnixFrom

		exit 0
	fi
fi

# Check archive or not.
MLCOUNT=
FILENUM=
if [ "$ARCHIVE" = "DO" -o $SUBSTYLE -eq 3 -o $SUBSTYLE -eq 4 -o "$COUNT" = "DO" ]
then
	MDIR=$SPOOL
	NDIR=
	while [ ! -d $MDIR ]
	do
		UDIR=`expr \
			${MDIR} : '\(/\)[^/]*//*$' \
			\| ${MDIR} : '\(.*[^/]\)//*[^/][^/]*//*$' \
			\| .`
		CDIR=`basename $MDIR`
		MDIR=$UDIR

		NDIR="$CDIR $NDIR"

		if [ -d $UDIR ]
		then
			break
		fi

		MDIR=$UDIR
	done

	for i in $NDIR
	do
		MDIR=$MDIR/$i
		mkdir $MDIR
	done

	if [ ! -f $SEQFILE ]
	then
		echo 0 > $SEQFILE
	else
		# get lock
		DELAY=`expr $MYPID % 6`

		# set delay time
		case "$DELAY" in
		0)	set 1 2 3 3 2 1 ;;
		1)	set 1 3 2 1 2 3 ;;
		2)	set 2 1 3 1 3 2 ;;
		3)	set 2 3 1 2 1 3 ;;
		4)	set 3 1 2 2 3 1 ;;
		*)	set 3 2 1 3 1 2 ;;
		esac

		umask 077
		until ln $SEQFILE $SEQLOCK >/dev/null 2>&1
		do
			DELAY=$1
			shift
			set $@ $DELAY
			sleep $DELAY
		done
		# lock complete

		umask 022

		expr `cat $SEQFILE` > /dev/null 2>&1
		if [ $? -ne 0 ]
		then
			ls -R $SPOOL/* | \
				egrep "[0-9][0-9][0-9][0-9][0-9][0-9][0-9]$"|\
				sort -nr | head -1 > $SEQFILE
		fi
		expr `cat $SEQFILE` > /dev/null 2>&1
		if [ $? -ne 0 ]
		then
			echo 0 > $SEQFILE
		fi
	fi

	expr "`cat $SEQFILE`" + 1 > $WORK
	cat $WORK > $SEQFILE

	SEQNUM=`cat $SEQFILE`
	FILENUM=`echo 000000$SEQNUM | sed -e 's/.*\(.......\)$/\1/'`
	
	rm -rf $SEQLOCK

	if [ "$COUNT" = "DO" ]
	then
		MLCOUNT="X-ML-Count: $FILENUM"
	fi
fi

# Put X-ML-Name.
if [ "$MLNAME" = "DO" ]
then
	XMLNAME="X-ML-Name: $ML"
fi

# Check change Reply-To:
if [ "$REPLYTO" ]
then
	REPLYTO=$REPLYTO
else
	REPLYTO=$ML@$DOMAIN
fi

# Make header
echo "Errors-To: $ML-request@$DOMAIN
$VHEAD
$LOOPCHECK" > $HEAD

if [ -f $DRIVER/$ML/$MYHEADER ]
then
	egrep '^X-' $DRIVER/$ML/$MYHEADER >> $HEAD
fi

XREPLY=0
case $REPLYSW in
1)
	echo "Reply-To: $REPLYTO" >> $HEAD
	XREPLY=1
	;;
2)
	set `sed -n -e '1,/^$/p' < $TEMP |
	     sed -e 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'|
	     egrep "^REPLY-TO:"` .

	if [ "$1" != "REPLY-TO:" ]
	then
		echo "Reply-To: $REPLYTO" >> $HEAD
		XREPLY=1
	fi
	;;
esac

if [ $XREPLY -eq 1 ]
then
	cat $TEMP > $WORK
	$AWK '
	/^$/, NR == 0 {print; next;}
	/^[Rr][Ee][Pp][Ll][Yy]-[Tt][Oo]:/ {print "X-"$0; next;}
	{print;}
	' < $WORK > $TEMP
fi

if [ "$SENDER" ]
then
	echo "Sender: $SENDER" >> $HEAD

	cat $TEMP > $WORK
	$AWK '
	/^$/, NR == 0 {print; next;}
	/^[Ss][Ee][Nn][Dd][Ee][Rr]:/ {print "X-"$0; next;}
	{print;}
	' < $WORK > $TEMP
fi

if [ "$PRECEDENCE" ]
then
	echo "Precedence: $PRECEDENCE" >> $HEAD

	cat $TEMP > $WORK
	$AWK '
	/^$/, NR == 0 {print; next;}
	/^[Pp][Rr][Ee][Cc][Ee][Dd][Ee][Nn][Cc][Ee]:/ {print "X-"$0; next;}
	{print;}
	' < $WORK > $TEMP
fi

if [ "$MLCOUNT" ]
then
	echo $MLCOUNT >> $HEAD
fi

if [ "$XMLNAME" ]
then
	echo $XMLNAME >> $HEAD
fi

if [ "$XHEADERS" ]
then
	cat $DRIVER/$ML/$XHEADERS |\
	$AWK '
	BEGIN {con = 0;}
	/^X-.*:/ {printf("echo \"%s\"\n", $0); con=1; next; }
	/^[ 	]/ { if(con == 1) {printf("echo \"%s\";\n", $0); next; } }
	{con = 0;}
	' > $WORK
	eval "`cat $WORK`" >> $HEAD
fi

# Get header and append to local header.
sed -n -e '1,/^$/p' < $TEMP >> $HEAD

# Get mail body.
sed -e '1,/^$/d' < $TEMP > $BODY

# Check mail body. If the mail dosenot have any text line, append information.
if [ ! "`$AWK '{ print $1 }' < $BODY`" ]
then
	echo "*** No text line in original.***" >> $BODY
fi

# Check subject.
egrep "^[Ss][Uu][Bb][Jj][Ee][Cc][Tt]:" $HEAD > $TEMP
if [ $? -eq 0 ]
then
	set `cat $TEMP` .
	if [ "$2" = "" ]
	then
		cat $HEAD > $TEMP
		echo "Subject: No title in original."		 > $HEAD
		egrep -v "^[Ss][Uu][Bb][Jj][Ee][Cc][Tt]:" $TEMP >> $HEAD
	fi
else
	cat $HEAD > $TEMP
	echo "Subject: No title in original."  > $HEAD
	cat $TEMP			      >> $HEAD
fi

if [ "$OTHERNAME" ]
then
	TAG=$OTHERNAME
else
	TAG=$ML
fi

case $SUBSTYLE in
2)
	cat $HEAD > $TEMP
	sed "1,/^$/ {
		s/[ 	]*\[$TAG\][ 	]*[Rr][Ee]://
		s/\[$TAG\][ 	]*//
		s/^[Ss][Uu][Bb][Jj][Ee][Cc][Tt]:/Subject: [$TAG]/
	}" < $TEMP > $HEAD
	;;
3)	
	cat $HEAD > $TEMP
	sed "1,/^$/ {
		s/[ 	]*\[$TAG:[0-9]*\][  	]*[Rr][Ee]://
		s/\[$TAG:[0-9]*\][ 	]*//
		s/^[Ss][Uu][Bb][Jj][Ee][Cc][Tt]:/Subject: [$TAG:$FILENUM]/
	}" < $TEMP > $HEAD
	;;
4)	
	cat $HEAD > $TEMP
	sed "1,/^$/ {
		s/[ 	]*\[$TAG:[0-9]*\][  	]*[Rr][Ee]://
		s/\[$TAG:[0-9]*\][ 	]*//
		s/^[Ss][Uu][Bb][Jj][Ee][Cc][Tt]:/Subject: [$TAG:$SEQNUM]/
	}" < $TEMP > $HEAD
	;;
esac

# Mail send to member of ML
cat $HEAD $BODY | $SENDMAIL -oi -f$UnixFrom -oem -odb $ML-dist

if [ "$ARCHIVE" = "DO" ]
then
	#
	# Chage spool dir. (after CML-3.1)
	#
	DELAY=`expr $MYPID % 6`

	# set delay time
	case "$DELAY" in
	0)	set 1 2 3 3 2 1 ;;
	1)	set 1 3 2 1 2 3 ;;
	2)	set 2 1 3 1 3 2 ;;
	3)	set 2 3 1 2 1 3 ;;
	4)	set 3 1 2 2 3 1 ;;
	*)	set 3 2 1 3 1 2 ;;
	esac
	umask 077
	until ln $SLKEY $SLOCK >/dev/null 2>&1
	do
		DELAY=$1
		shift
		set $@ $DELAY
		sleep $DELAY
	done
	# lock complete

	umask 022

	#D=`find $SPOOL -type d -name "[0-9][0-9][0-9][0-9]000" -print`
	D=`ls -d $SPOOL/[0-9][0-9][0-9][0-9]000/. 2>/dev/null`
	if [ "$D" = "" ]
	then
		I=0
		while :
		do
			D=`echo 0000${I}000 | sed -e 's/.*\(.......\)$/\1/'`
			F=`echo 0000${I}??? | sed -e 's/.*\(.....\)$/\1/'`

			FF=`find $SPOOL -type f -name "$F" -print`
			if [ "$FF" = "" ]
			then
				break
			fi

			if [ ! -d $SPOOL/$D.d ]
			then
				mkdir $SPOOL/$D.d
			fi

			for i in $FF
			do
				mv $i $SPOOL/${D}.d/00`basename $i`
			done

			mv $SPOOL/${D}.d $SPOOL/$D
			I=`expr $I + 1`
		done
	fi
	rm -rf $SLOCK

	DATE=`$AWK '
		BEGIN {rcv = 0;}
		/^$/, NR == 0 { next;}
		/^From / {next;}
		/^[Dd][Aa][Tt][Ee]:/ {rcv = 1; print; next;}
		/^[ 	]/ && rcv == 1 {print; next;}
		{rcv = 0};' < $HEAD | sed -e 's/[Dd][Aa][Tt][Ee]: //'`

	D="`expr $FILENUM / 1000`000"
	D=`echo 0000000$D | sed -e 's/.*\(.......\)$/\1/'`

	if [ ! -d $SPOOL/$D ]
	then
		mkdir $SPOOL/$D
	fi

	echo "From $UnixFrom $DATE"  > $SPOOL/$D/$FILENUM
	cat $HEAD $BODY | b64decode >> $SPOOL/$D/$FILENUM

	if [ "$KEEP" = "ALL" ]
	then
		KEEP=0
	fi

	if [ $KEEP -gt 0 ]
	then
		RM=`expr $FILENUM - $KEEP`
		MAX=`echo 000000$RM | sed -e 's/.*\(.......\)$/\1/'`

		MIN=`ls -R $SPOOL/* |\
			egrep "[0-9][0-9][0-9][0-9][0-9][0-9][0-9]$" | \
			sort -n | head -1`
		MIN=`basename $MIN`

		I=$MIN
		while [ $I -lt $MAX ]
		do

			CUR=`echo 0000000$I | sed -e 's/.*\(.......\)$/\1/'`
			D="`expr $CUR / 1000`000"
			D=`echo 0000000$D | sed -e 's/.*\(.......\)$/\1/'`
			if [ -s $SPOOL/$D/$CUR ]
			then
				rm -rf $SPOOL/$D/$CUR
			fi
			I=`expr $I + 1`
		done
	fi

fi

$GREP DGST $LIST 2>&1 > /dev/null
if [ $? -eq 0 ]
then
	# Get lock.
	DELAY=`expr $MYPID % 6`

	# Get delay time.
	case "$DELAY" in
	0)	set 1 2 3 3 2 1 ;;
	1)	set 1 3 2 1 2 3 ;;
	2)	set 2 1 3 1 3 2 ;;
	3)	set 2 3 1 2 1 3 ;;
	4)	set 3 1 2 2 3 1 ;;
	*)	set 3 2 1 3 1 2 ;;
	esac

	umask 077
	until ln $LKEY $DGSTLOCK >/dev/null 2>&1
	do
		DELAY=$1
		shift
		set $@ $DELAY
		echo "sleep $DELAY"
		sleep $DELAY
	done
	# Lock comprele.
	umask 022

	if [ ! -f $DGSTFILE ]
	then
		echo "-------- Start Digest of $ML ML --------" >> $DGSTFILE
		echo ""                                         >> $DGSTFILE
	else

		echo ""                                         >> $DGSTFILE
		echo "--------"                                 >> $DGSTFILE
		echo ""                                         >> $DGSTFILE
	fi
	cat $HEAD $BODY | sed -e 's/^-/- -/' | b64decode        >> $DGSTFILE

	if [ ! -f $DRIVER/$ML/cmdt.pid ]
	then
		$DRIVER/cmdt $ML >/dev/null 2>&1 & 
	fi

	rm -f $DGSTLOCK
fi

exit 0
